package cn.estsh.i3plus.ext.mes.api.base.bu;

import cn.estsh.i3plus.ext.mes.pojo.bean.MesBusiConfig;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import io.swagger.annotations.ApiOperation;

import java.util.Map;

/**
 * @Author: xinwang.yi
 * @CreateDate: 2021/01/19 15:06 下午
 * @Description:
 **/
public interface IBusiConfigService {

    /**
     * 按条件分页查询业务配置表信息
     *
     * @param busiConfig 查询条件
     * @param pager 分页条件
     * @return 业务配置表分页数据
     */
    @ApiOperation(value = "按条件分页查询业务配置表信息", notes = "按条件分页查询业务配置表信息")
    ListPager<MesBusiConfig> queryBusiConfigByPager(MesBusiConfig busiConfig, Pager pager);

    /**
     * 新增业务配置表信息
     * @param busiConfig 业务配置表信息
     * @return 业务配置表数据
     */
    @ApiOperation(value = "新增业务配置表信息", notes = "新增业务配置表信息")
    MesBusiConfig insertBusiConfig(MesBusiConfig busiConfig);

    /**
     * 更新业务配置表信息
     * @param busiConfig 业务配置表信息
     * @return 业务配置表数据
     */
    @ApiOperation(value = "更新业务配置表信息", notes = "更新业务配置表信息")
    MesBusiConfig updateBusiConfig(MesBusiConfig busiConfig);

    /**
     * 根据id启用或禁用业务配置表信息
     * @param id 业务配置表信息ID
     * @param status 状态
     * @param organizeCode 组织代码
     * @param userName 操作人
     * @return 结果
     */
    @ApiOperation(value = "根据id启用或禁用业务配置表信息", notes = "根据id启用或禁用业务配置表信息")
    int updateBusiConfigValidById(Long id, Integer status, String organizeCode, String userName);

    /**
     * 根据配置代码查询业务配置信息
     * @param organizeCode 组织代码
     * @param cfgCode 配置代码
     * @return 业务配置信息
     */
    @ApiOperation(value = "根据配置代码查询业务配置信息", notes = "根据配置代码查询业务配置信息")
    MesBusiConfig getBusiConfigByCfgCode(String organizeCode, String cfgCode);

    /**
     * 获取配置信息MAP
     * @param organizeCode 组织代码
     * @param cfgCode 配置代码
     * @return 配置信息MAP
     */
    @ApiOperation(value = "获取配置信息MAP", notes = "获取配置信息MAP")
    Map<String, String> getConfigMap(String organizeCode, String cfgCode);

}
