package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IConsistencyAffirmPartService;
import cn.estsh.i3plus.ext.mes.api.base.IConsistencyAffirmPartService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesConsistencyAffirmPart;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: xiangming.liao
 * @CreateDate: 2021/01/18 13:33 下午
 * @Description:
 **/
@RestController
@Api(tags = "一致性确认物料")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/consistency-affirm-part")
public class ConsistencyAffirmPartController {
    public static final Logger LOGGER = LoggerFactory.getLogger(ConsistencyAffirmPartController.class);

    @Autowired
    private IConsistencyAffirmPartService consistencyAffirmPartService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增一致性确认物料", notes = "新增一致性确认物料")
    public ResultBean insertConsistencyAffirmPart(@RequestBody MesConsistencyAffirmPart consistencyAffirmPart) {
        try {
            consistencyAffirmPart.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            consistencyAffirmPartService.insertConsistencyAffirmPart(consistencyAffirmPart);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {
            LOGGER.error(imppException.getErrorMsg() + "：{}", imppException.getErrorDetail(), imppException);
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "更新一致性确认物料", notes = "更新一致性确认物料")
    public ResultBean updateConsistencyAffirmPart(@RequestBody MesConsistencyAffirmPart consistencyAffirmPart) {
        try {

            if (consistencyAffirmPart.getId() == null || consistencyAffirmPart.getId().longValue() < 1) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            consistencyAffirmPartService.updateConsistencyAffirmPart(consistencyAffirmPart);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/query")
    @ApiOperation(value = "按条件分页查询一致性确认物料", notes = "按条件分页查询一致性确认物料")
    public ResultBean queryConsistencyAffirmPartByPager(MesConsistencyAffirmPart consistencyAffirmPart, Pager pager) {
        try {
            consistencyAffirmPart.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("查询成功").setListPager(consistencyAffirmPartService.queryConsistencyAffirmPartByPager(consistencyAffirmPart,pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update-status-batch")
    @ApiOperation(value = "批量禁用/启用详情", notes = "批量禁用/启用详情")
    public ResultBean updateConsistencyAffirmPartValidByIds(String[] idsStr, int status) {
        try {
            ConvertBean.modelSafeArrayNumber(idsStr, true);
            consistencyAffirmPartService.updateConsistencyAffirmPartValidByIds(idsStr,status);
            return ResultBean.success("修改成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/status-update")
    @ApiOperation(value = "禁用/启用详情", notes = "禁用/启用详情")
    public ResultBean updateConsistencyAffirmPartValidById(Long id, int status) {
        try {
            if (StringUtils.isEmpty(id)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            consistencyAffirmPartService.updateConsistencyAffirmPartValidById(id, status, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
