package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IPrinterCfgService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesPrinterCfg;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "打印机配置控制层")
public class PrinterCfgController extends MesBaseController {

    @Autowired
    private IPrinterCfgService printerCfgService;

    @GetMapping(value = "/printer-cfg/query")
    @ApiOperation(value = "按条件分页查询打印机配置信息")
    public ResultBean queryPrinterCfgByPager(MesPrinterCfg printerCfg, Pager pager) {
        try {
            printerCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(printerCfgService.queryPrinterCfgByPager(printerCfg, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/printer-cfg/insert")
    @ApiOperation(value = "新增打印机配置信息")
    public ResultBean insertPrinterCfg(MesPrinterCfg printerCfg) {
        try {
            ValidatorBean.beginValid(printerCfg)
                    .notNull(MesExtConstWords.PRINTER_NO, printerCfg.getPrinterNo())
                    .notNull(MesExtConstWords.PRINTER_NAME, printerCfg.getPrinterName());

            printerCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            printerCfg.setCreateUser(AuthUtil.getSessionUser().getUserName());
            printerCfgService.insertPrinterCfg(printerCfg);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/printer-cfg/update")
    @ApiOperation(value = "更新打印机配置信息")
    public ResultBean updatePrinterCfg(MesPrinterCfg printerCfg) {
        try {
            ValidatorBean.beginValid(printerCfg)
                    .notNull(MesExtConstWords.ID, printerCfg.getId())
                    .notNull(MesExtConstWords.PRINTER_NO, printerCfg.getPrinterNo())
                    .notNull(MesExtConstWords.PRINTER_NAME, printerCfg.getPrinterName());

            printerCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            printerCfg.setModifyUser(AuthUtil.getSessionUser().getUserName());
            printerCfgService.updatePrinterCfg(printerCfg);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/printer-cfg/status-update")
    @ApiOperation(value = "根据id启用或禁用打印机配置信息")
    public ResultBean updatePrinterCfgValidById(Long id, Integer status) {
        try {
            if (StringUtils.isEmpty(id)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            printerCfgService.updatePrinterCfgValidById(id, status, AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
