package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IFourMManageService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFourMManage;
import cn.estsh.i3plus.ext.mes.pojo.model.FourMManageModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesFourMManageRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.sqlpack.MesHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/2/2 15:19
 **/
@Service
public class FourMManageService implements IFourMManageService {

    @Autowired
    private MesFourMManageRepository fourMManageRepository;

    /**
     * 新增
     *
     * @param equipmentParam
     * @return
     */
    @Override
    public MesFourMManage insert(MesFourMManage equipmentParam) {
        ConvertBean.modelInitialize(equipmentParam, AuthUtil.getSessionUser());
        equipmentParam.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentParam.getManageCode(), "manageCode", ddlPackBean);
        MesFourMManage mesEquipmentCheck = fourMManageRepository.getByProperty(ddlPackBean);
        if (!ObjectUtils.isEmpty(mesEquipmentCheck)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
        }

        return fourMManageRepository.insert(equipmentParam);
    }

    /**
     * 修改
     *
     * @param equipmentParam
     * @return
     */
    @Override
    public MesFourMManage update(MesFourMManage equipmentParam) {
        //1、判断单据是否存在
        Long id = equipmentParam.getId();
        MesFourMManage equipmentParam2 = fourMManageRepository.getById(id);
        if (ObjectUtils.isEmpty(equipmentParam2)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "自动配置不存在");
        }

        if (!equipmentParam.getManageCode().equals(equipmentParam2.getManageCode())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(equipmentParam.getManageCode(), "manageCode", ddlPackBean);
            DdlPreparedPack.getNumEqualPack(equipmentParam.getManageType(), "manageType", ddlPackBean);
            MesFourMManage mesEquipmentCheck = fourMManageRepository.getByProperty(ddlPackBean);
            if (!ObjectUtils.isEmpty(mesEquipmentCheck)) {
                throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
            }
        }
        equipmentParam2.setManageType(equipmentParam.getManageType());
        equipmentParam2.setManageCode(equipmentParam.getManageCode());
        equipmentParam2.setManageContent(equipmentParam.getManageContent());
        ConvertBean.serviceModelUpdate(equipmentParam2, AuthUtil.getSessionUser().getUserName());
        return fourMManageRepository.save(equipmentParam2);
    }

    /**
     * 查询
     *
     * @param equipmentParam
     * @param pager
     * @return
     */
    @Override
    public ListPager<FourMManageModel> queryByPager(MesFourMManage equipmentParam, Pager pager) {
        List<FourMManageModel> modelList = new ArrayList<>();
        DdlPackBean packBean = MesHqlPack.getMesAllData(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(equipmentParam.getManageCode(), "manageCode", packBean);
        DdlPreparedPack.getNumEqualPack(equipmentParam.getManageType(), "manageType", packBean);
        DdlPreparedPack.getNumEqualPack(equipmentParam.getIsValid(), "isValid", packBean);
        pager = PagerHelper.getPager(pager, fourMManageRepository.findByHqlWhereCount(packBean));
        List<MesFourMManage> fourMManageList = fourMManageRepository.findByHqlWherePage(packBean, pager);
        if (!CollectionUtils.isEmpty(fourMManageList)) {
            for (MesFourMManage item : fourMManageList) {
                if (null == item) {
                    continue;
                }
                FourMManageModel model = new FourMManageModel();
                BeanUtils.copyProperties(item, model);
                model.setManageTypeName(MesExtEnumUtil.MANAGE_TYPE.valueOfDescription(model.getManageType()));
                modelList.add(model);
            }
        }
        return new ListPager(modelList, pager);
    }

    /**
     * 根据
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public int updateIsValidById(long id, Integer status) {
        //1、判断单据是否存在
        MesFourMManage equipmentParam = fourMManageRepository.getById(id);
        if (ObjectUtils.isEmpty(equipmentParam)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "自动配置不存在");
        }

        return fourMManageRepository.updateByProperties(new String[]{"id"}, new Object[]{id},
                new String[]{MesConstWords.IS_VALID, "modifyUser", "modifyDatetime"},
                new Object[]{status, AuthUtil.getSessionUser().getUserName(), TimeTool.getNowTime(true)});
    }


    /**
     * 异常
     *
     * @param softTypeCode          softTypeCode
     * @param errorDetailString     错误消息字符串
     * @param imppExceptionEnumCode 异常代码
     * @param args                  参数
     */
    private void throwImppException(String softTypeCode, String errorDetailString, String imppExceptionEnumCode, Object... args) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(softTypeCode)
                .setErrorCode(imppExceptionEnumCode)
                .setErrorDetail(errorDetailString, args)
                .build();
    }
}
