package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessCell;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterExt;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProcessCellRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProcess;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.repository.MesProcessRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCellRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 文件描述
 * 工序工位信息导入
 *
 * @author xiangming.liao
 * @date 2021年04月12日 13:48
 */
@Service
public class ProcessCellExcelService implements IExcelImportExtService {


    @Autowired
    private MesProcessCellRepository processCellRepository;

    @Autowired
    private MesWorkCellRepository workCellRepository;

    @Autowired
    private MesProcessRepository processRepository;

    @Autowired
    private MesWorkCenterExtRepository workCenterExtRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 工序工位信息导入
     *
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesProcessCell(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesProcessCell(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 工序工位信息导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesProcessCell(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesProcessCell集合
        List<MesProcessCell> processCellExtList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的 生产线代码+工位 唯一校验
        Map<String, String> processCellByOnlyMap = new HashMap<>();

        //根据生产线代码、工位代码查询中是否存在数据
        List<MesProcessCell> processCellList = processCellRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});

        Map<String, List<MesProcessCell>> processCellMap = CollectionUtils.isEmpty(processCellList) ? null :
                processCellList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode()));


        // 查询产线表
        Map<String, Object> workCenterCodeEntityMap;
        try {
            workCenterCodeEntityMap = MesCommonUtil.getCodeEntityMap(workCenterExtRepository, MesExtConstWords.WORK_CENTER_CODE, organizeCode, "产线表");
        } catch (Exception e) {
            workCenterCodeEntityMap = null;
        }

        // 查询工位表
        List<MesWorkCell> workCellList = workCellRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesWorkCell>> workCellCodeEntityMap = CollectionUtils.isEmpty(workCellList) ? null :
                workCellList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode()));

        // 查询工序表
        Map<String, Object> processCodeEntityMap;
        try {
            processCodeEntityMap = MesCommonUtil.getCodeEntityMap(processRepository, MesExtConstWords.PROCESS_CODE, organizeCode, "工序表");
        } catch (Exception e) {
            processCodeEntityMap = null;
        }

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //生产线代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String workCenterCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                String workCenterName = null;

                //工位代码
                String workCellCode = null;
                String workCellName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    workCellCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //工序代码
                String processCode = null;
                String processName = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    processCode = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }


                //校验生产线代码
                if (CollectionUtils.isEmpty(workCenterCodeEntityMap) || !workCenterCodeEntityMap.containsKey(workCenterCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                } else {
                    workCenterName = ((MesWorkCenterExt) workCenterCodeEntityMap.get(workCenterCode)).getWorkCenterName();
                }

                //校验工位代码
                if (StringUtils.isEmpty(workCellCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (StringUtils.isEmpty(errorInfo)) {
                        if (CollectionUtils.isEmpty(workCellCodeEntityMap) || !workCellCodeEntityMap.containsKey(workCenterCode + "&" + workCellCode)) {
                            errorNum++;
                            cellNum += "A+B;";
                            errorInfo += "第A+B列数据无效;";
                        } else {
                            workCellName = workCellCodeEntityMap.get(workCenterCode + "&" + workCellCode).get(0).getWorkCellName();
                        }
                    }
                }

                //校验工序代码
                if (StringUtils.isEmpty(processCode)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else if (!processCodeEntityMap.containsKey(processCode)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据无效;";
                } else {
                    processName = ((MesProcess) processCodeEntityMap.get(processCode)).getProcessName();
                }

                //校验唯一性 不重复
                if (!processCellByOnlyMap.containsKey(workCenterCode + "&" + workCellCode)) {
                    processCellByOnlyMap.put(workCenterCode + "&" + workCellCode, String.valueOf(rowNum));
                    //根据生产线代码、工位代码查询中是否存在数据

                    if (!CollectionUtils.isEmpty(processCellMap) && processCellMap.containsKey(workCenterCode + "&" + workCellCode)) {
                        errorNum++;
                        cellNum += "A+B;";
                        errorInfo += "第A+B列数据已经存在数据库;";
                    }
                } else {
                    errorNum++;
                    cellNum += "A+B;";
                    errorInfo += "第A+B列数据跟第" + processCellByOnlyMap.get(workCenterCode + "&" + workCellCode) +
                            "行的第A+B列数据重复;";
                }

                //没有错误的时候，MesProcessCell
                if (errorNum == 0) {
                    MesProcessCell mesProcessCell = new MesProcessCell();
                    mesProcessCell.setWorkCenterCode(workCenterCode);
                    mesProcessCell.setWorkCenterName(workCenterName);
                    mesProcessCell.setWorkCellCode(workCellCode);
                    mesProcessCell.setWorkCellName(workCellName);
                    mesProcessCell.setProcessCode(processCode);
                    mesProcessCell.setProcessName(processName);
                    mesProcessCell.setOrganizeCode(organizeCode);
                    ConvertBean.serviceModelInitialize(mesProcessCell, userName);
                    processCellExtList.add(mesProcessCell);
                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultExtModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultExtModel.setExcelList((failRowNum > 0) ? null : processCellExtList);
        return excelImportResultExtModel;
    }

    /**
     * 工序工位信息导入-数据入库
     *
     * @param excelImportResultExtModel
     */
    private void insertExcelMesProcessCell(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesProcessCell> processCellList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(processCellList)) {
                for (MesProcessCell processCell : processCellList) {
                    processCellRepository.insert(processCell);
                }
            }
        }
    }
}
