package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFourMManage;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesFourMManageRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 4M管理导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxFourMManageExcelService implements IExcelImportExtService {

    @Autowired
    private MesFourMManageRepository fourMManageRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 4M管理导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesFourMManage(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesFourMManage(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 4M管理导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesFourMManage(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesFourMManage集合
        List<MesFourMManage> fourMManageList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的项目管理代码唯一校验，key：manageCode，value：row
        Map<String, String> manageCodeMap = new HashMap<>();

        //查询4M管理表
        List<MesFourMManage> fourMManages = fourMManageRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesFourMManage>> fourMManageMap = CollectionUtils.isEmpty(fourMManages) ? null :
                fourMManages.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesFourMManage::getManageCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //项目管理代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String manageCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //管理类型
                String manageType = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    manageType = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //项目管理内容
                String manageContent = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    manageContent = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验项目管理代码不重复
                if (!manageCodeMap.containsKey(manageCode)) {
                    manageCodeMap.put(manageCode, String.valueOf(rowNum));
                    //校验 项目管理代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(fourMManageMap) && fourMManageMap.containsKey(manageCode)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + manageCodeMap.get(manageCode) + "行的第A列数据重复;";
                }

                //校验管理类型
                if (StringUtils.isEmpty(manageType)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                   if (StringUtils.isEmpty(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType))) {
                       errorNum ++;
                       cellNum += "B;";
                       errorInfo += "第B列数据无效;";
                   }
                }

                //校验项目管理内容
                if (StringUtils.isEmpty(manageContent)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else {
                    if (manageContent.length() > 255) {
                        errorNum ++;
                        cellNum += "C;";
                        errorInfo += "第C列数据长度不能超过255个字符;";
                    }
                }

                //没有错误的时候，封装MesFourMManage
                if (errorNum == 0) {
                    MesFourMManage fourMManage;
                    if(isExist){
                        fourMManage = fourMManageMap.get(manageCode).get(0);
                        ConvertBean.serviceModelUpdate(fourMManage, userName);
                    }else{
                        fourMManage = new MesFourMManage();
                        ConvertBean.serviceModelInitialize(fourMManage, userName);
                    }
                    fourMManage.setManageCode(manageCode);
                    fourMManage.setManageContent(manageContent);
                    fourMManage.setManageType(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType));
                    fourMManage.setOrganizeCode(organizeCode);

                    fourMManageList.add(fourMManage);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : fourMManageList);
        return excelImportResultModel;
    }

    /**
     * 4M管理扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesFourMManage(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesFourMManage> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesFourMManage fourMManage : excelList) {
                    if(StringUtils.isEmpty(fourMManage.getId())){
                        fourMManageRepository.insert(fourMManage);
                    }else{
                        fourMManageRepository.save(fourMManage);
                    }
                }
            }
        }
    }
}
