package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesPartExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesPartContainerCapacity;
import cn.estsh.i3plus.pojo.mes.repository.MesContainerTypeRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesPartContainerCapacityRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 产品容器容量配置导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxPartContainerCapacityExcelService implements IExcelImportExtService {

    @Autowired
    private MesContainerTypeRepository containerTypeRepository;

    @Autowired
    private MesPartExtRepository partExtRepository;

    @Autowired
    private MesPartContainerCapacityRepository partContainerCapacityRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 产品容器容量配置导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractPartContainerCapacity(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelPartContainerCapacity(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 产品容器容量配置导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractPartContainerCapacity(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesPartContainerCapacity
        List<MesPartContainerCapacity> partContainerCapacityList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的物料代码+容器类型代码唯一校验，key：partNo&ctCode，value：row
        Map<String, String> existMap = new HashMap<>();

        // 查询物料表
        Map<String, Object> partCodeEntityMap;
        try {
            partCodeEntityMap = MesCommonUtil.getCodeEntityMap(partExtRepository, MesExtConstWords.PART_NO, organizeCode, "物料表");
        } catch (Exception e) {
            partCodeEntityMap = null;
        }

        // 容器类型表
        Map<String, Object> ctCodeEntityMap;
        try {
            ctCodeEntityMap = MesCommonUtil.getCodeEntityMap(containerTypeRepository, MesExtConstWords.CT_CODE, organizeCode, "容器类型表");
        } catch (Exception e) {
            ctCodeEntityMap = null;
        }

        //查询产品容器容量配置表
        List<MesPartContainerCapacity> partContainerCapacities = partContainerCapacityRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesPartContainerCapacity>> partContainerCapacityMap = CollectionUtils.isEmpty(partContainerCapacities) ? null :
                partContainerCapacities.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> (o.getPartNo() + "&" + o.getCtCode())));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //容器代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String partNo = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //容器类型
                String ctCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    ctCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //容量
                String ctQty = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    ctQty = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                if (CollectionUtils.isEmpty(partCodeEntityMap) || !partCodeEntityMap.containsKey(partNo)) {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                }

                //校验容器类型
                if (StringUtils.isEmpty(ctCode)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (CollectionUtils.isEmpty(ctCodeEntityMap) || !ctCodeEntityMap.containsKey(ctCode)) {
                        errorNum ++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                boolean isExist = false;
                //校验物料代码+容器类型代码不重复
                if (!existMap.containsKey(partNo + "&" + ctCode)) {
                    existMap.put(partNo + "&" + ctCode, String.valueOf(rowNum));
                    //校验 物料代码+容器类型 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(partContainerCapacityMap) && partContainerCapacityMap.containsKey(partNo + "&" + ctCode)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A+B;";
                    errorInfo += "第A+B列数据跟第" + existMap.get(partNo + "&" + ctCode) + "行的第A+B列数据重复;";
                }

                //校验容量
                if (StringUtils.isEmpty(ctQty)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else {
                    try {
                        Integer.valueOf(ctQty);
                    } catch (NumberFormatException e) {
                        errorNum ++;
                        cellNum += "C;";
                        errorInfo += "第C列数据无效;";
                    }
                }

                //没有错误的时候，封装MesPartContainerCapacity
                if (errorNum == 0) {
                    MesPartContainerCapacity partContainerCapacity;
                    if(isExist){
                        partContainerCapacity = partContainerCapacityMap.get(partNo + "&" + ctCode).get(0);
                        ConvertBean.serviceModelUpdate(partContainerCapacity, userName);
                    }else{
                        partContainerCapacity = new MesPartContainerCapacity();
                        ConvertBean.serviceModelInitialize(partContainerCapacity, userName);
                    }
                    partContainerCapacity.setPartNo(partNo);
                    partContainerCapacity.setCtCode(ctCode);
                    partContainerCapacity.setCtQty(Integer.valueOf(ctQty));
                    partContainerCapacity.setSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());
                    partContainerCapacity.setOrganizeCode(organizeCode);

                    partContainerCapacityList.add(partContainerCapacity);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : partContainerCapacityList);
        return excelImportResultModel;
    }

    /**
     * 产品容器容量配置导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelPartContainerCapacity(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesPartContainerCapacity> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesPartContainerCapacity partContainerCapacity : excelList) {
                    if(StringUtils.isEmpty(partContainerCapacity.getId())){
                        partContainerCapacityRepository.insert(partContainerCapacity);
                    }else{
                        partContainerCapacityRepository.save(partContainerCapacity);
                    }
                }
            }
        }
    }
}
