package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.jx;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesManageCode;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterExt;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesManageCodeRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description : 管理码导入
 * @Reference :
 * @Author : xinwang.yi
 * @CreateDate : 2023-10-23
 * @Modify:
 **/
@Service
public class ManageCodeExcelService implements IExcelImportExtService {

    @Autowired
    private MesManageCodeRepository manageCodeRepository;

    @Autowired
    private MesWorkCenterExtRepository workCenterExtRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 管理码配置导入
     *
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesManageCode(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesManageCode(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 管理码配置导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesManageCode(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesManageCode集合
        List<MesManageCode> mesManageCodeList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的 管理码 唯一校验
        Map<String, String> manageCodeByOnlyMap = new HashMap<>();

        //根据管理码 查询替代料中是否存在数据
        List<MesManageCode> manageCodeList = manageCodeRepository.findByProperty(
                new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, MesManageCode> manageCodeMap = CollectionUtils.isEmpty(manageCodeList) ? null :
                manageCodeList.stream().filter(o -> null != o).collect(Collectors.toMap(MesManageCode::getManageCode, Function.identity(), (x,y) -> y));

        List<MesWorkCenterExt> workCenterList = workCenterExtRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED, MesExtConstWords.IS_VALID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue()});
        Map<String, MesWorkCenterExt> workCenterCodeMap = CollectionUtils.isEmpty(workCenterList) ? null :
                workCenterList.stream().filter(o -> null != o).collect(Collectors.toMap(MesWorkCenterExt::getWorkCenterCode, Function.identity(), (x,y) -> y));



        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //管理码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String manageCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();


                //产线代码
                String workCenterCode = null;
                String workCenterName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    workCenterCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验管理码
                if (!StringUtils.isEmpty(manageCode)) {
                    if (!manageCodeMap.containsKey(manageCode)) {
                        errorNum++;
                        cellNum += "A;";
                        errorInfo += "第A列数据无效;";
                    }
                } else {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据必填;";
                }

                //产线代码
                if (StringUtils.isEmpty(workCenterCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else if ((CollectionUtils.isEmpty(workCenterCodeMap)) || (!workCenterCodeMap.containsKey(workCenterCode))){
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据无效;";
                } else {
                    workCenterName = workCenterCodeMap.get(workCenterCode).getWorkCenterName();
                }

                String key = manageCode;

                Boolean isExist = false;

                //校验唯一性 不重复
                if (!manageCodeByOnlyMap.containsKey(key)) {
                    manageCodeByOnlyMap.put(key, String.valueOf(rowNum));
                    //根据管理码查询替代信息中是否存在数据

                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(manageCodeMap) && manageCodeMap.containsKey(key)) {
                        errorNum++;
                        cellNum += "A;";
                        errorInfo += "第A列数据已经存在数据库;";
                    }

                    //校验是否已经存在数据库
                    if (!CollectionUtils.isEmpty(manageCodeMap) && manageCodeMap.containsKey(key)) isExist = true;
                } else {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + manageCodeByOnlyMap.get(key) +
                            "行的第A列数据重复;";
                }

                //没有错误的时候，封装MesManageCode
                if (errorNum == 0) {
                    MesManageCode mesManageCode;
                    if (isExist) {
                        mesManageCode = manageCodeMap.get(key);
                        ConvertBean.serviceModelUpdate(mesManageCode, userName);

                    } else {
                        mesManageCode = new MesManageCode();
                        ConvertBean.serviceModelInitialize(mesManageCode, userName);
                        mesManageCode.setManageCode(manageCode);

                    }

                    mesManageCode.setWorkCenterCode(workCenterCode);
                    mesManageCode.setWorkCenterName(workCenterName);
                    mesManageCode.setSystemSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());

                    mesManageCodeList.add(mesManageCode);

                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultExtModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultExtModel.setExcelList((failRowNum > 0) ? null : mesManageCodeList);
        return excelImportResultExtModel;
    }

    /**
     * 管理码导入-数据入库
     *
     * @param excelImportResultExtModel
     */
    private void insertExcelMesManageCode(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesManageCode> processTorqueManageList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(processTorqueManageList)) manageCodeRepository.saveAll(processTorqueManageList);
        }
    }
}
